@@ -1,5 +1,4 @@
-import argparse
-from ml_investment.data import SF1Data
+from ml_investment.data_loaders.sf1 import SF1BaseData
 from ml_investment.download import QuandlDownloader
 from ml_investment.utils import load_config, save_json
 
@@ -21,14 +20,14 @@ def main():
     downloader.zip_download('datatables/SHARADAR/TICKERS?qopts.export=true',
                             '{}/tickers.zip'.format(config['sf1_data_path']))
 
-    data_loader = SF1Data(config['sf1_data_path'])
-    ticker_list = data_loader.load_base_data()['ticker'].unique().tolist()
+    base_df = SF1BaseData(config['sf1_data_path']).load()
+    tickers = base_df['ticker'].unique().tolist()
     
-    downloader.ticker_download('datatables/SHARADAR/SF1?ticker={ticker}', ticker_list, 
+    downloader.ticker_download('datatables/SHARADAR/SF1?ticker={ticker}', tickers, 
                                save_dirpath='{}/core_fundamental'.format(config['sf1_data_path']), 
                                skip_exists=False,  batch_size=10, n_jobs=4)
 
-    downloader.ticker_download('datatables/SHARADAR/DAILY?ticker={ticker}', ticker_list, 
+    downloader.ticker_download('datatables/SHARADAR/DAILY?ticker={ticker}', tickers, 
                                save_dirpath='{}/daily'.format(config['sf1_data_path']), 
                                skip_exists=False, batch_size=5, n_jobs=4)
 
